
/*
===========================================================================
 Filemane	: TinyTxDMX.c
 Function	: DMX transmission test set
 Compiler  	: Atmel Studio 6.0.1843
 MCU       	: Tiny2313
 XTAL      	: 20 MHz
 Creation	: 12/06/2012 - V1.0

 Copyright Matthew Edwards, matt@lomae.com

 DMX Transmit routines based on code written
 by Hendrik Hlscher, www.hoelscher-hi.de

 Hardware
 	Tiny 2313 runing at 20Mhz
 	Hundreds BCD connected for PD4 - PD6 
 	Tens BCD connected for PB0 - PB3 
 	Units BCD connected for PB4 - PB7 
 	Wake up switch on PD2 (Int0)
 	Mode selection on PD0
 	Power/Flash LED on PD3

 ===========================================================================
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see http://www.gnu.org/licenses/
 ===========================================================================
*/

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h> 
#include "sbit.h" 

#define F_OSC 20000L		

#define IBG   		(10)				//interbyte gap [us]

#define KEY0            SBIT( PIND, 2 )
#define KEY0_PULLUP     SBIT( PORTD, 2 )

#define LED0            SBIT( PORTD, 3 )
#define LED0_DDR        SBIT( DDRD, 3 )
#define Flash 10
#define Time2Sleep 20

volatile uint8_t	 DmxField[100];		//array of DMX vals

uint8_t 	gCurDmxCh;					//current DMX channel
uint8_t		gDmxState;
uint16_t	StartDmx;

uint8_t		tick;
uint8_t		tickRIP;

void sleepNow()
{
	LED0 = 0;					// LED off (high)
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
	sleep_enable();
	sleep_mode();
}

ISR(INT0_vect)
{
	LED0 = 1;								// LED on (low)
	tick = 0;
	tickRIP = 0;
	sleep_disable();						//wake up
}




enum {BREAK, STARTB, DATA};

ISR (USART_TX_vect) {
	uint8_t DmxState= gDmxState;

	if (DmxState == BREAK) {
		UBRRH  = 0;
		UBRRL  = (F_OSC/1600);					//90.9kbaud
		UDR    = 0;								//send break
		gDmxState= STARTB;
	}
	else if (DmxState == STARTB) {
		UBRRH  = 0;
		UBRRL  = ((F_OSC/4000)-1);				//250kbaud
		UDR    = 0;								//send start byte
		gDmxState= DATA;
		gCurDmxCh= 0;
	}
	else {
		_delay_us(IBG);
		uint16_t CurDmxCh= gCurDmxCh;
		UDR= DmxField[CurDmxCh++];				//send data
		if (CurDmxCh == sizeof(DmxField)) gDmxState= BREAK; //new break if all ch sent
		else gCurDmxCh= CurDmxCh;
	}
}

void ioinit (void) 	{
	DDRB  = 0b00000000;   //portb as input
	PORTB = 0b11111111;	 // enable internal pull up resistors

	DDRD  = 0b00001010;  // Bits 1 and 3 are outputs
    PORTD = 0b11110101; // enable internal pulll up resistors

	GIMSK|=(1<<INT0);						//External Interrupt Request 0 Enable
	//	MCUCR|=(1<<ISC01);						//Any Logical change of INT0 generates an interrupt request.
	
	//	TCCR1B |= ((1 << CS10) | (1 << CS11));	// Set up timer at Fcpu/1
	//	TCCR1B |= ((1 << CS10) | (1 << CS11));	// Set up timer at Fcpu/64
	TCCR1B |=  (1 << CS12);					// Set up timer at Fcpu/256
	//	TCCR1B |= ((1 << CS10) | (1 << CS12));	// Set up timer at Fcpu/1024
	
	tick = 0;
	tickRIP = 0;


	UBRRH = 0;
	UBRRL = ((F_OSC/4000)-1);			//250kbaud, 8N2
	UCSRC |= (1<<UCSZ1)|(1<<UCSZ0)|(1<<USBS);
	UCSRB |= (1<<TXEN)|(1<<TXCIE);
	UDR   = 0;							//start USART

	//Data
	gDmxState= BREAK;					//start with break
	}

int main (void)
{
	char units, tens, hundreds;
	ioinit();
	sei();

	while(1)
	{
	
/*	if( KEY0 == 0 ) 
	{					// if key pressed (low)
		LED0 = 1;						// LED on (low)
	}	
	else
	LED0 = 0;							// LED off (high)
*/	
		// BCD Switch definitions
		/* Units
			Sw 1 - PB4 correct position shift left 4
			Sw 2 - PB6 shift left 5
			Sw 4 - PB7 shift left 5
			Sw 8 - PB5 shift left 2	*/
		units = ((~PINB & 0x80)>>5)|((~PINB & 0x40)>>5)|((~PINB & 0x20)>>2)|((~PINB & 0x10)>>4); 
		/* Tens
			Sw 1 - PB0 correct position
			Sw 2 - PB2 shift left 1
			Sw 4 - PB3 shift left 1
			Sw 8 - PB1 shift right 2 */
		tens = ((~PINB & 0x02)<<2)|((~PINB & 0x08)>>1)|((~PINB & 0x04)>>1)|((~PINB & 0x01)); 
		/* Hundreds
			Sw 1 - PD4 shift left 4
			Sw 2 - PD6 shift left 5
			Sw 4 - PD5 shift left 3
			Sw 8 - Not Used */
		hundreds = 0;
		hundreds = (((~PIND & 0x20)>>3)|((~PIND & 0x40)>>5)|((~PIND & 0x10)>>4)); 
		if (hundreds > 5) hundreds = 5;		// Trim hundreds to less than or equal 5
		StartDmx = 100*hundreds+10*tens+units;
		
		if (StartDmx < 1) StartDmx = 1;
		
		if (StartDmx > 512) StartDmx = 512;
				
//		StartDmx = StartDmx + 1;

		if (bit_is_set(PIND, PD0)) 
		{
			if (StartDmx<256) 
			{	
				DmxField[0] = StartDmx & 0xff;
				DmxField[1] = 0x00;
				DmxField[2] = 0x00;
			}
			else 
			{
				DmxField[0] = StartDmx-256;
				DmxField[1] = 255;
				DmxField[2] = 0x00;
			}				
		}
		else 
		{
			DmxField[0] = units & 0x0f;
			DmxField[1] = tens & 0x0f;
			DmxField[2] = hundreds & 0x07;
		}

		// Check timer value in if statement, true when count matches 1/5 of a second
		// Target Timer Count = (Input Frequency / Prescale) / Target Frequency - 1
		// Target Timer Count = 20000000/256/5 -1)
		// Target Timer Count = 15625
		
		if (TCNT1 >= 15625)
		{
			if( tick == Flash )
			{								// if key pressed (low)
				LED0 = 1;					// LED on (low)
				tick = 0;
				tickRIP ++;
			}
			else
			{
				LED0 = 0;					// LED off (high)
				tick ++;
//				tickRIP ++;
			}
			TCNT1 = 0;						// Reset timer value
		}
		if(tickRIP == Time2Sleep) 
		{
			sleepNow();
		}
		
	}
}
